#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-07-30
#
# monitor Check Point Eventia analyzer correlation unit
#
# 2016-08-04: changed invetory from index to ip address (index was changing ??)
# 2018-01-05: fixed inventory function
# 2018-03-12: code cleanup
# 2020-06-07: added parse function
#             code cleanup
# 2020-06-08: changed snmp-scan function
# 2021-08-14: rewritten for CMK 2.0
#
# sample string_table
# [['1', '192.167.10.10', '1469790660', '898864', '189144']]
#
import time
from dataclasses import dataclass
from typing import Optional, Dict

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    check_levels,
    State,
    SNMPTree,
    all_of,
    startswith,
    any_of,
    equals,
    get_rate,
    GetRateError,
    get_value_store,
    IgnoreResultsError,
    render,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


@dataclass
class CheckpointEvaCu:
    cunumeventsrcvd: int
    cuconnectionduration: int
    last: str
    since: str


def parse_checkpoint_eva_cu(string_table: StringTable) -> Dict[str, CheckpointEvaCu]:
    section = {}
    for cu in string_table:
        try:
            cuindex, cuip, culastrcvdtime, cunumeventsrcvd, cuconnectionduration = cu
        except ValueError:
            continue

        # last = time.strftime('%c', time.localtime(int(culastrcvdtime)))
        # since = time.strftime('%c', time.localtime(time.time() - cuconnectionduration))

        section[cuip] = CheckpointEvaCu(
            cunumeventsrcvd=int(cunumeventsrcvd),
            cuconnectionduration=int(cuconnectionduration),
            last=time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time() - int(cunumeventsrcvd))),
            since=time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time() - int(cuconnectionduration))),
        )

    return section


def discovery_checkpoint_eva_cu(section: Dict[str, CheckpointEvaCu]) -> DiscoveryResult:
    for key in section.keys():
        yield Service(item=key)


def check_checkpoint_eva_cu(item, section: Dict[str, CheckpointEvaCu]) -> Optional[CheckResult]:
    try:
        cu = section[item]
    except KeyError:
        return

    now_time = time.time()
    value_store = get_value_store()
    try:
        cunumeventsrcvd = get_rate(
            value_store,
            f'checkpoint_eva_cu_cunumeventsrcvd_{item}',
            now_time,
            cu.cunumeventsrcvd,
            raise_overflow=False
        )
    except GetRateError:
        raise IgnoreResultsError('Initializing counters')

    yield from check_levels(
        value=cu.cuconnectionduration,
        label='Uptime',
        metric_name='checkpoint_eva_cu_uptime',
        render_func=render.timespan
    )

    yield from check_levels(
        value=cunumeventsrcvd,
        label='Events',
        metric_name='checkpoint_eva_cu_events',
        render_func=lambda v: f'{v:.0f}/s'
    )

    yield Result(state=State.OK, notice=f'Up since {cu.since}')
    yield Result(state=State.OK, notice=f'Last event at: {cu.last}')


register.snmp_section(
    name='checkpoint_eva_cu',
    parse_function=parse_checkpoint_eva_cu,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.25.1.9.1',  # CHECKPOINT-MIB::cpsemdCorrelationUnitEntry
        oids=[
            '1',  # cpsemdCorrelationUnitIndex
            '2',  # cpsemdCorrelationUnitIP
            '3',  # cpsemdCorrelationUnitLastRcvdTime
            '4',  # cpsemdCorrelationUnitNumEventsRcvd
            '5',  # cpsemdConnectionDuration
        ]
    ),
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_eva_cu',
    service_name='Correlation unit %s',
    discovery_function=discovery_checkpoint_eva_cu,
    check_function=check_checkpoint_eva_cu,
)
