#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# date  : 2016-07-29
#
# monitor Check Point Eventia analyzer
#
# 2018-03-12: code cleanup
# 2020-06-07: split checkpoint_eva in to checkpoint_eva_status and checkpoint_eva_cu_status
#             code cleanup
# 2020-06-08: changed snmp-scan function
# 2021-08-13: rewritten for CMK 2.0
#
# sample info SamrtEvent activated
# [['1', '2305546', '7606198', '1469790658', '0', '', '']]
#
# sample info SmartEvent not localy running
# [['1', '0', '0', '', '2', 'CPSEMD not running', 'CPSEMD not running - process appears to be down']]
#
# sample info SmartEvent not activated)
# [['0', '0', '0', '', '2', 'CPSEMD not running', 'CPSEMD not running - process appears to be down']]
#

import time
from dataclasses import dataclass
from typing import Optional

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    check_levels,
    State,
    SNMPTree,
    all_of,
    startswith,
    any_of,
    equals,
    not_equals,
    get_rate,
    GetRateError,
    get_value_store,
    IgnoreResultsError,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


@dataclass
class CheckpointEvaStatus:
    procalive: Optional[int]
    neweventshandled: int
    updateshandled: int
    lasteventtime: Optional[int]
    statcode: Optional[int]
    statshortdescr: str
    statlongdescr: str


def parse_checkpoint_eva_status(string_table: StringTable) -> Optional[CheckpointEvaStatus]:
    try:
        procalive, neweventshandled, updateshandled, lasteventtime, statcode, statshortdescr, statlongdescr = \
            string_table[0]
    except (IndexError, ValueError):
        return

    return CheckpointEvaStatus(
        procalive=int(procalive) if procalive.isdigit() else None,
        neweventshandled=int(neweventshandled),
        updateshandled=int(updateshandled),
        lasteventtime=time.strftime('%Y-%m-%d %H:%M:%S',
                                    time.localtime(int(lasteventtime))) if lasteventtime.isdigit() else None,
        statcode=int(statcode) if statcode.isdigit() else None,
        statshortdescr=statshortdescr,
        statlongdescr=statlongdescr,
    )


def discovery_checkpoint_eva_status(section: CheckpointEvaStatus) -> DiscoveryResult:
    yield Service()


def check_checkpoint_eva_status(section: CheckpointEvaStatus) -> CheckResult:
    now_time = time.time()
    value_store = get_value_store()
    raise_ingore_res = False

    for key, value, label in [
        ('neweventshandled', section.neweventshandled, 'New events'),
        ('updateshandled', section.updateshandled, 'Updates'),
    ]:
        try:
            value = get_rate(value_store, f'cisccheckpoint_eva_status_{key}', now_time, value, raise_overflow=False)
        except GetRateError:
            raise_ingore_res = True
        yield from check_levels(
            value=value,
            label=label,
            metric_name=f'checkpoint_eva_{key}',
            render_func=lambda v: f'{v:0.2f}/s',
        )

    if raise_ingore_res:
        raise IgnoreResultsError('Initializing counters')

    if section.lasteventtime:
        yield from check_levels(
            value=section.lasteventtime,
            label='Last event at',
            render_func=lambda v: f'{v}',
        )

    if section.procalive != 1:
        yield Result(state=State.CRIT, summary='Process not alive')

    if section.statcode != 0:
        yield Result(state=State.CRIT, summary=section.statshortdescr)

    if len(section.statlongdescr) != 0:
        yield Result(state=State.OK, notice=f'Status long: {section.statlongdescr}')


register.snmp_section(
    name='checkpoint_eva_status',
    parse_function=parse_checkpoint_eva_status,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.25.1',  # CHECKPOINT-MIB::cpsemd
        oids=[
            '1',  # cpsemdProcAlive
            '2',  # cpsemdNewEventsHandled
            '3',  # cpsemdUpdatesHandled
            '4',  # cpsemdLastEventTime
            '101',  # cpsemdStatCode
            '102',  # cpsemdStatShortDescr
            '103',  # cpsemdStatLongDescr
        ]
    ),
    detect=all_of(
        any_of(
            startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
            all_of(
                equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
                equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
            )
        ),
        not_equals('.1.3.6.1.4.1.2620.1.25.1.101.0', '2')
    )
)

register.check_plugin(
    name='checkpoint_eva_status',
    service_name='Eventia analyzer status',
    discovery_function=discovery_checkpoint_eva_status,
    check_function=check_checkpoint_eva_status,
)
